package cn.com.qmth.stmms.ms.admin.service;

import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.Work;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.repository.MarkSubjectRepo;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import cn.com.qmth.stmms.ms.core.repository.WorkRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 评卷工作服务
 */
@Service
public class WorkService {

    @Autowired
    private WorkRepo workRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    /**
     * 删除评卷工作
     *
     * @param workId
     */
    @Transactional
    public void delete(Long workId) {
        Long paperCount = paperRepo.countByWorkIdAndTest(workId, TrialEnum.DEFAULT.getId());
        if (paperCount > 0) {
            throw new RuntimeException("该评卷工作已有数据，不能删除");
        }
        workRepo.delete(workId);

    }

    /**
     * 创建评卷工作
     *
     * @param work
     */
    @Transactional
    public void save(Work work) {
        workRepo.save(work);
        Subject[] subjects = Subject.values();
        for (Subject subject : subjects) {
            MarkSubject markSubject = new MarkSubject(subject, work.getId());
            markSubject.setWorkId(work.getId());
            markSubjectRepo.save(markSubject);
        }
    }

}
