package cn.com.qmth.stmms.ms.core.repository;

import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTask;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Query;

import java.util.List;

/**
 * Created by zhengmin on 2016/9/23.
 */
public interface MarkTaskRepo extends JpaRepository<MarkTask, Long>, JpaSpecificationExecutor {

    int countByMarkerIdAndPaperIdAndWorkIdAndRandomSeqNew(Long markerId, Long paperId, Long randomSeqNew, Long workId);

    /**
     * 查询指定阶段试卷的评卷任务
     *
     * @param paperId 试卷id
     * @param stage   评卷阶段
     * @return
     */
    List<MarkTask> findByPaperIdAndStage(Long paperId, MarkStage stage);

    /**
     * 评卷员任务数量
     *
     * @param markerId
     * @return
     */
    long countByMarkerId(Long markerId);

    long countByMarkerIdAndQuestionId(Long markerId, Long questionId);

    long countByQuestionIdIn(List<Long> questionIds);

    @Query(value = "select count(1) from mark_task m left join paper p on m.paper_id = p.id where m.question_id = ? and p.batch_no = (select max(batch_no) from paper where question_id = ?) and m.marker_id = ?", nativeQuery = true)
    long countByQuestionId(Long questionId, Long questionId1, Long markerId);

    int countByQuestionIdAndStageAndResultIsNull(Long questionId, MarkStage stage);

    @Query(value = "select count(1) from mark_task m left join paper p on m.paper_id = p.id where m.question_id = ?1 and m.stage = ?2 and m.result = ?3 and p.batch_no = (select max(batch_no) from paper where question_id = ?1)", nativeQuery = true)
    int countByQuestionIdAndStageAndResult(Long questionId, int stage, String result);

    int countByQuestionIdAndMarkerIdAndStageAndResultIsNull(Long questionId, Long markerId, MarkStage stage);

    @Query(value = "select count(1) from mark_task m left join paper p on m.paper_id = p.id where m.question_id = ?1 and m.marker_id = ?2 and m.stage = ?3 and m.result = ?4 and p.batch_no = (select max(batch_no) from paper where question_id = ?5)", nativeQuery = true)
    int countByQuestionIdAndMarkerIdAndStageAndResult(Long questionId, Long markerId, int stage, String result, Long questionId1);

    int countByWorkIdAndSubjectAndMarkerIdAndStageAndResult(Long workId, Subject subject, Long markerId, MarkStage stage, String result);

    int countByWorkIdAndSubjectAndMarkerIdAndStageAndQuestionIdAndResult(Long workId, Subject subject, Long markerId, MarkStage stage, Long questionId, String result);

    int countByWorkIdAndSubjectAndMarkerIdAndStage(Long workId, Subject subject, Long markerId, MarkStage stage);

    int countByWorkIdAndSubjectAndMarkerIdAndStageAndQuestionId(Long workId, Subject subject, Long markerId, MarkStage stage, Long questionId);

    /**
     * 查询评卷员的评卷任务
     *
     * @param markerId
     * @param stage
     * @return
     */
    @Query(value = "select mt from MarkTask mt left outer join fetch mt.paper where mt.markerId = ? and mt.stage = ?")
    List<MarkTask> findByMarkerIdAndStage(Long markerId, MarkStage stage);

    /**
     * 查询评卷员的评卷任务
     *
     * @param markerId
     * @param stage
     * @param questionId
     * @return
     */
    @Query(value = "select mt from MarkTask mt left outer join fetch mt.paper where mt.markerId = ? and mt.stage = ? and mt.questionId = ? order by mt.updatedOn desc")
    List<MarkTask> findByMarkerIdAndStageAndQuestionId(Long markerId, MarkStage stage, Long questionId);

    /**
     * 删除评卷员所有任务
     *
     * @param subject
     * @param stage
     * @return
     */
    Long deleteByWorkIdAndSubjectAndStage(Long workId, Subject subject, MarkStage stage);

    /**
     * 删除科目下所有任务
     *
     * @param workId
     * @param subject
     * @return
     */
    Long deleteByWorkIdAndSubjectAndTest(Long workId, Subject subject, int test);

    /**
     * 统计评卷员各试题已评数及总数（分档阶段）
     *
     * @param markerId
     * @return
     */
    @Query(value = "SELECT q.`id`,q.`name`,SUM(IF(t.`result` IS NULL,1,0)),COUNT(t.question_id), q.area_code \n" +
            "FROM mark_task t LEFT OUTER JOIN exam_question q ON t.`question_id` = q.`id` LEFT  JOIN paper p ON t.paper_id = p.id WHERE " +
            "t.`marker_id` = ? and t.stage = ? and p.is_missing = false and p.is_test = ?  GROUP BY t.`question_id` ORDER BY q.`id`", nativeQuery = true)
    List<Object[]> countGroupByQuestion(Long markerId, Integer stageId, int test);

    /**
     * 统计评卷员各试题已评数及总数（分档阶段）
     *
     * @param markerId
     * @return
     */
    @Query(value = "SELECT q.`id`,q.`name`,SUM(IF(t.`result` IS NULL,1,0)),COUNT(t.question_id), q.area_code \n" +
            "FROM mark_task t LEFT OUTER JOIN exam_question q ON t.`question_id` = q.`id` LEFT  JOIN paper p ON t.paper_id = p.id WHERE " +
            "t.`marker_id` = ? and t.stage = ? and p.is_missing = false GROUP BY t.`question_id` ORDER BY q.`id`", nativeQuery = true)
    List<Object[]> countGroupByQuestion(Long markerId, Integer stageId);


    /**
     * 统计评卷员指定试题的各档位数量
     *
     * @param questionIds
     * @param markerId
     * @return
     */
    @Query(value = "SELECT t.result,COUNT(*) FROM mark_task t " +
            "WHERE t.`question_id` = ? AND t.`marker_id` = ? and t.stage = 1 " +
            "GROUP BY t.`result` ORDER BY t.result", nativeQuery = true)
    List<Object[]> countGroupByQuestionIdAndLevel(Long questionId, Long markerId);

    //2019.10.16修改为所有试卷
    @Query(value = "SELECT t.result,COUNT(*) FROM mark_task t " +
            "WHERE t.`question_id` in (?) and t.stage = 1 " +
            "GROUP BY t.`result`,t.question_id ORDER BY t.result", nativeQuery = true)
    List<Object[]> countGroupByQuestionIdAndLevel(List<Long> questionId);

    @Query(value = "SELECT t.result,COUNT(*) FROM mark_task t " +
            "WHERE t.`marker_id` = ? and t.stage = 1 " +
            "GROUP BY t.`result` ORDER BY t.result", nativeQuery = true)
    List<Object[]> countGroupByLevel(Long markerId);


    @Query(value = "SELECT COUNT(*) FROM mark_task t " +
            "WHERE t.`marker_id` = ? and t.stage = 1 and t.result = ? ", nativeQuery = true)
    int countByLevelAndResult(Long markerId, String result);
    //2019.10.16修改为所有老师的评卷
//    @Query(value = "SELECT COUNT(*) FROM mark_task t " +
//            "WHERE t.`question_id` in (?) and t.stage = 1 and t.result = ? ", nativeQuery = true)
//    int countByLevelAndResult(Long questionId, String result);

    @Query(value = "SELECT COUNT(*) FROM mark_task t " +
            "WHERE t.`marker_id` = ? and t.stage = 1 and t.result is null", nativeQuery = true)
    int countByLevel(Long markerId);
    //2019.10.16修改为所有老师的评卷
//    @Query(value = "SELECT COUNT(*) FROM mark_task t " +
//            "WHERE t.`question_id` in (?) and t.stage = 1 and t.result is null", nativeQuery = true)
//    int countByLevel(Long questionId);

    int countByQuestionIdInAndStageAndResult(List<Long> questionIds, MarkStage stage, String result);

    int countByQuestionIdInAndStageAndResultIsNull(List<Long> questionIds, MarkStage stage);


    /**
     * 统计指定试卷的所有任务数量
     *
     * @param id
     * @return
     */
    Long countByPaperId(Long id);

    int countByWorkIdAndSubjectAndStageAndResult(Long workId, Subject markSubject, MarkStage level, String code);

    int countByWorkIdAndSubjectAndStage(Long workId, Subject markSubject, MarkStage level);

    int countByWorkIdAndSubjectAndStageAndQuestionIdAndResult(Long workId, Subject subject, MarkStage level, Long id, String code);

    int countByWorkIdAndSubjectAndStageAndQuestionId(Long workId, Subject subject, MarkStage level, Long id);

    List<MarkTask> findByWorkIdAndSecretNumber(Long workId, String secretNumber);

    @Query(value = "SELECT max(t.randomSeq) FROM MarkTask t " +
            "WHERE t.workId = ?1 and t.subject = ?2 and t.markerId = ?3 and t.stage = ?4 and t.questionId = ?5 ")
    Integer findRandomSeqByWorkIdAndSubjectAndMarkerIdAndStageAndQuestionId(Long workId, Subject subject, Long markerId, MarkStage stage, Long questionId);

    @Query(value = "SELECT count(1) from mark_task m left join paper p on m.paper_id = p.id where m.question_id = ?1 and m.marker_id = ?2 and m.stage = ?3 and m.result is null and p.is_missing = ?4 and p.batch_no = (select max(batch_no) from paper where question_id = ?5)", nativeQuery = true)
    int countByQuestionIdAndMarkerIdAndStageAndResultIsNullAndIsMissing(Long questionId, Long markerId, int stage, boolean isMissing, Long questionId1);

    @Query(value = "SELECT count(1) from mark_task m left join paper p on m.paper_id = p.id where m.question_id = ?1 and m.stage = ?2 and m.result =?3 and p.is_missing = ?4 and p.batch_no = (select max(batch_no) from paper where question_id = ?5)", nativeQuery = true)
    int countByQuestionIdAndStageAndResultAndIsMissing(Long questionId, int stage, String result, boolean isMissing, Long questionId1);

    MarkTask findByPaperIdAndSecretNumber(Long paperId, String secretNumber);
}
