package cn.com.qmth.stmms.ms.marking.api;

import cn.com.qmth.stmms.ms.commons.utils.SqlUtil;
import cn.com.qmth.stmms.ms.core.domain.Level;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.user.MarkRight;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.assembler.LevelStatAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.MarkerAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.QuestionStatAssembler;
import cn.com.qmth.stmms.ms.marking.dto.LevelStatDTO;
import cn.com.qmth.stmms.ms.marking.dto.MarkerDTO;
import cn.com.qmth.stmms.ms.marking.dto.QuestionStatDTO;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 评卷中，评卷员的相关数据api
 * Created by zhengmin on 2016/10/14.
 */
@RestController
@RequestMapping("api/markers")
public class MakrerApi {
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(MakrerApi.class);

    @Autowired
    private MarkUserRepo markUserRepo;

    @Autowired
    private MarkTaskRepo markTaskRepo;

    @Autowired
    private MarkerAssembler markerAssembler;

    @Autowired
    private QuestionStatAssembler questionStatAssembler;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private LevelStatAssembler levelStatAssembler;

    @Autowired
    ExamQuestionRepo examQuestionRepo;

    @Autowired
    SqlUtil sqlUtil;

    @Autowired
    PaperRepo paperRepo;

    /**
     * 评卷员信息
     *
     * @param markerId 评卷员用户id
     * @return
     */
    @RequestMapping(value = "{markerId}", method = RequestMethod.GET)
    public MarkerDTO getMarker(@PathVariable Long markerId) {
        MarkUser marker = markUserRepo.findOne(markerId);
        return markerAssembler.toDTO(marker);
    }

    /**
     * 评卷员列表
     *
     * @param workId  评卷工作id
     * @param subject 科目
     * @return
     */
    @RequestMapping(method = RequestMethod.GET)
    public List<MarkUser> list(@RequestParam Long workId,
                               @RequestParam Subject subject) {
        List<MarkUser> markUsers = markUserRepo.findByWorkIdAndSubjectAndRole(workId, subject, Role.MARKER);
        String markSubjectId = workId + "-" + subject.toString();
        MarkSubject markSubject = markSubjectRepo.findOne(markSubjectId);
        if (MarkStage.INIT == markSubject.getStage()) {
            markUsers = markUsers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_LEVELING || m.getMarkRight() == MarkRight.ALLOW_ALL).collect(Collectors.toList());
        } else if (MarkStage.LEVEL == markSubject.getStage()) {
            markUsers = markUsers.stream().filter(m -> m.getMarkRight() == MarkRight.ALLOW_SCORING || m.getMarkRight() == MarkRight.ALLOW_ALL).collect(Collectors.toList());
        }
        return markUsers;
    }

    /**
     * 评卷员分档数量及占比数据
     *
     * @param marker     评卷员用户id
     * @param questionId 试题id
     * @return
     */
    @RequestMapping(value = "{marker}/stat/levels", method = RequestMethod.GET)
    public List<LevelStatDTO> levelStats(@PathVariable MarkUser marker, @RequestParam Long questionId) throws Exception {
        List<LevelStatDTO> levelStatDTOs = new ArrayList<>();
        MarkSubject markSubject = markSubjectRepo.findOne(marker.getWorkId() + "-" + marker.getSubject().toString());
        List<Level> levels = levelRepo.findByWorkId(markSubject.getWorkId());
        Map<String, Level> levelMap = levels.stream().collect(Collectors.toMap(Level::getCode, l -> l));
        //统计workId下各考点的数量
        paperRepo.countGroupByLevel(questionId, questionId)
                .forEach(o -> {
                    LevelStatDTO levelStatDTO = levelStatAssembler.toDTO(o);
                    levelStatDTO.setGcount(levelStatDTO.getCount());
                    if (Objects.isNull(levelStatDTO.getId())) {
                        //求任务数为null的条数
//                        int count = markTaskRepo.countByQuestionIdAndMarkerIdAndStageAndResultIsNull(questionId, marker.getId(), MarkStage.LEVEL);
                        int count = markTaskRepo.countByQuestionIdAndMarkerIdAndStageAndResultIsNullAndIsMissing(questionId, marker.getId(), MarkStage.LEVEL.ordinal(), false, questionId);
                        levelStatDTO.setCount(count);
                    } else {
//                        int count = markTaskRepo.countByQuestionIdAndStageAndResult(questionId, MarkStage.LEVEL, levelStatDTO.getId().toString());
                        int count = markTaskRepo.countByQuestionIdAndStageAndResultAndIsMissing(questionId, MarkStage.LEVEL.ordinal(), levelStatDTO.getId().toString(), false, questionId);
                        levelStatDTO.setCount(count);
                    }
                    levelStatDTOs.add(levelStatDTO);
                });

        for (Level level : levels) {
            //所有档位依次比较
            long count = levelStatDTOs.stream().filter(l -> String.valueOf(l.getId()).equals(level.getCode())).count();
            if (count == 0) {
                LevelStatDTO dto = new LevelStatDTO();
                dto.setId(level.getCode());
                dto.setCount(0);
                dto.setPercent(0.0);

                //当前老师当前档位评档次数(所有考试)
                int countNew = markTaskRepo.countByQuestionIdAndStageAndResult(questionId, MarkStage.LEVEL.ordinal(), level.getCode());
                dto.setCount(countNew);
                levelStatDTOs.add(dto);
            }
        }

        //当前老师当前试卷的评档次数(不分档位)
        long kdtotal = markTaskRepo.countByQuestionId(questionId, questionId, marker.getId());
        //当前老师所有的评档次数(不分档位)
        long total = paperRepo.countByWorkIdAndQuestionId(markSubject.getWorkId(), questionId);

        levelStatDTOs.forEach(o -> {
            if (o.getId() != null) {
                o.setPt(levelMap.get(o.getId()).getPt());
                o.setKdpt(levelMap.get(o.getId()).getKdpt());
                o.setCount(Objects.isNull(o.getCount()) ? 0 : o.getCount());
                double p = (double) o.getCount() / kdtotal;
                BigDecimal bd = new BigDecimal(p).setScale(3, RoundingMode.HALF_EVEN);
                o.setPercent(bd.doubleValue());

                o.setGcount(Objects.isNull(o.getGcount()) ? 0 : o.getGcount());
                double gp = (double) o.getGcount() / total;
                BigDecimal gbd = new BigDecimal(gp).setScale(3, RoundingMode.HALF_EVEN);
                o.setGpercent(gbd.doubleValue());

                int count = markTaskRepo.countByQuestionIdAndMarkerIdAndStageAndResult(questionId, marker.getId(), MarkStage.LEVEL.ordinal(), o.getId().toString(), questionId);
                o.setPercent(count == 0 ? 0 : o.getPercent());
                o.setCount(count);
            }
        });
        return levelStatDTOs;
    }

    /**
     * 各试题统计数据
     *
     * @param marker 评卷员用户id
     * @return
     */
    @RequestMapping(value = "{marker}/stat/questions", method = RequestMethod.GET)
    public List<QuestionStatDTO> questions(@PathVariable MarkUser marker) {
        List<QuestionStatDTO> questionStatDTOs = new ArrayList<>();
        List<Object[]> qStats = null;
        MarkSubject markSubject = markSubjectRepo.findOne(marker.getWorkId() + "-" + marker.getSubject().toString());
        if (markSubject.getStage() == MarkStage.SCORE) {
            qStats = markTaskRepo.countGroupByQuestion(marker.getId(), markSubject.getStage().ordinal());
        } else {
            qStats = markTaskRepo.countGroupByQuestion(marker.getId(), markSubject.getStage().ordinal(), markSubject.getTest());
        }
        if (qStats != null) {
            for (Object[] objects : qStats) {
                QuestionStatDTO dto = questionStatAssembler.toDTO(objects);
                questionStatDTOs.add(dto);
            }
        }
        return questionStatDTOs;
    }

}
