package cn.com.qmth.stmms.ms.marking.api;

import cn.com.qmth.stmms.ms.core.domain.MarkLog;
import cn.com.qmth.stmms.ms.core.domain.MarkLogOperType;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.Work;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTask;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.MarkLogRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskRepo;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import cn.com.qmth.stmms.ms.core.repository.WorkRepo;
import cn.com.qmth.stmms.ms.marking.assembler.PaperAssembler;
import cn.com.qmth.stmms.ms.marking.dto.PaperDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * @author by xf
 * @date 2019/11/5.
 */
@RestController
@RequestMapping("/api/task")
public class TaskApi {

    @Autowired
    private PaperAssembler paperAssembler;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private MarkTaskRepo markTaskRepo;

    @Autowired
    private MarkLogRepo markLogRepo;

    @Autowired
    private WorkRepo workRepo;

    /**
     * 任务管理查询
     *
     * @param workId
     * @param secretNumber
     * @param type
     * @return
     */
    @RequestMapping("/search")
    public PaperDTO searchBySecretNumber(@RequestParam("workId") Long workId,
                                         @RequestParam("secretNumber") String secretNumber,
                                         @RequestParam("type") String type) {
        PaperDTO paperDTO = null;
        if ("paper".equals(type)) {
            Specification<Paper> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("workId"), workId));
                if (secretNumber != null) {
                    predicates.add(builder.equal(root.get("secretNumber"), secretNumber));
                }

                return builder.and(predicates.toArray(new Predicate[predicates.size()]));
            };
            Optional<Paper> paperOptional = paperRepo.findAll(specification, new PageRequest(0, 1))
                    .getContent().stream().findFirst();
            if (paperOptional.isPresent()) {
                paperDTO = paperAssembler.toDTO(paperOptional.get());
            }
        } else if ("task".equals(type)) {
            Specification<MarkTask> specification = (root, query, builder) -> {
                List<Predicate> predicates = new ArrayList<>();
                predicates.add(builder.equal(root.get("workId"), workId));
                if (secretNumber != null) {
                    predicates.add(builder.equal(root.get("secretNumber"), secretNumber));
                }
                return builder.and(predicates.toArray(new Predicate[predicates.size()]));
            };
            Optional<MarkTask> taskOptional = markTaskRepo.findAll(specification, new PageRequest(0, 1))
                    .getContent().stream().findFirst();
            if (taskOptional.isPresent()) {
                paperDTO = paperAssembler.toDTO(taskOptional.get().getPaper());
                //设置为任务的密号
                paperDTO.setSn(taskOptional.get().getSecretNumber());
            }
        }
        return paperDTO;

    }

    /**
     * 重评
     *
     * @param paperId
     * @param secretNumber
     * @return
     */
    @Transactional
    @RequestMapping("/reevaluation")
    public boolean reevaluation(@RequestParam("paperId") Long paperId,
                                @RequestParam("secretNumber") String secretNumber) {
        Paper paper = paperRepo.findOne(paperId);
        paper.reject(null);
        //重评后撤销标准卷
//        paper.setSample(false);
        MarkTask markTask = markTaskRepo.findByPaperIdAndSecretNumber(paperId, secretNumber);
        if (markTask != null) {
            markTask.setRejected(true);
            markTask.setOriginLevel(markTask.getResult());
            markTask.setResult(null);
        }
        markTaskRepo.save(markTask);
        paperRepo.save(paper);

        //加入重评日志
        String operResult = "-";
        MarkLog markLogPrev = markLogRepo.findTopByCreateUserIdAndOperTypeAndSubjectAndExamNumberOrderByCreateTimeDesc(markTask.getMarkerId(), MarkLogOperType.REEVALUATION_LEVEl.getId(), markTask.getSubject().toString(), paper.getExamNumber());
        if (Objects.nonNull(markLogPrev)) {
            operResult = Optional.ofNullable(markLogPrev.getOperDataAfter()).orElse("-");
        }
        Work work = workRepo.findOne(paper.getWorkId());
        MarkLog markLog = new MarkLog(markTask.getMarkerId(), markTask.getMarkerName(), Role.MARKER, markTask.getSubject(), paper.getExamNumber(), paper.getStudentName(), MarkLogOperType.REEVALUATION_LEVEl.getId(), paper.getWorkId(), paper.getId(), markTask.getStage(), operResult, markTask.getOriginLevel(), MarkLogOperType.REEVALUATION_LEVEl.getName(), work.getName());
        markLogRepo.save(markLog);
        return true;
    }

}
