package cn.com.qmth.stmms.ms.marking.assembler;

import cn.com.qmth.stmms.ms.commons.config.ImageConfig;
import cn.com.qmth.stmms.ms.commons.config.SystemConfig;
import cn.com.qmth.stmms.ms.commons.constant.SystemConstant;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.Student;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTask;
import cn.com.qmth.stmms.ms.core.repository.StudentRepo;
import cn.com.qmth.stmms.ms.marking.dto.MarkTaskDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * Created by zhengmin on 2016/10/12.
 */
@Component
public class MarkTaskAssembler {

    @Autowired
    private SystemConfig systemConfig;

    @Autowired
    ImageConfig imageConfig;

    @Autowired
    StudentRepo studentRepo;

    public MarkTaskDTO toDTO(MarkTask markTask) {

        MarkTaskDTO markTaskDTO = null;
        if (markTask != null) {
            markTaskDTO = new MarkTaskDTO();
            markTaskDTO.setId(markTask.getId());
            markTaskDTO.setRejected(markTask.isRejected());
            markTaskDTO.setMarkerId(markTask.getMarkerId());
            markTaskDTO.setMarker(markTask.getMarkerName());
            markTaskDTO.setUpdatedOn(markTask.getUpdatedOn());
            markTaskDTO.setResult(markTask.getResult());
            Paper paper = markTask.getPaper();
            if (markTask.getStage() == MarkStage.LEVEL) {
                markTaskDTO.setLevel(markTask.getResult());
                markTaskDTO.setOriginLevel(markTask.getOriginLevel());
            } else {
                markTaskDTO.setLevel(paper.getLevel());
            }
            markTaskDTO.setSn(markTask.getSecretNumber());
            markTaskDTO.setRedoLevel(paper.getRedoLevel());
            String imgSrc = null;
            String thumbSrc = null;
            if (imageConfig.isImageEnc()) {
                Student student = studentRepo.findByWorkIdAndExamNumberAndTest(paper.getWorkId(), paper.getExamNumber(), String.valueOf(TrialEnum.DEFAULT.getId()));
                imgSrc = systemConfig.getImageUrl(paper.getWorkId(), paper.getSubject().ordinal(), student.getId(), SystemConstant.IMAGE);
                thumbSrc = systemConfig.getThumbUrl(paper.getWorkId(), paper.getSubject().ordinal(), student.getId(), SystemConstant.THUMB);
            } else {
                imgSrc = systemConfig.getImageUrl(markTask.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
                thumbSrc = systemConfig.getThumbUrl(markTask.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
            }
//            String imgSrc = systemConfig.getImageUrl(markTask.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
//            String thumbSrc = systemConfig.getThumbUrl(markTask.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
            markTaskDTO.setThumbSrc(thumbSrc);
            markTaskDTO.setImgSrc(imgSrc);
            markTaskDTO.setSample(paper.isSample());
            markTaskDTO.setMarkByLeader(paper.isMarkByLeader());
            markTaskDTO.setRandomSeqNew(markTask.getRandomSeqNew());
        }
        return markTaskDTO;
    }

    public MarkTaskDTO toDTO(MarkTask markTask, boolean oldRejected) {
        MarkTaskDTO markTaskDTO = null;
        if (markTask != null) {
            markTaskDTO = new MarkTaskDTO();
            markTaskDTO.setId(markTask.getId());
            markTaskDTO.setRejected(markTask.isRejected());
            markTaskDTO.setOldRejected(oldRejected);
            markTaskDTO.setMarker(markTask.getMarkerName());
            markTaskDTO.setUpdatedOn(markTask.getUpdatedOn());
            markTaskDTO.setResult(markTask.getResult());
            Paper paper = markTask.getPaper();
            if (markTask.getStage() == MarkStage.LEVEL) {
                markTaskDTO.setLevel(markTask.getResult());
                markTaskDTO.setOriginLevel(markTask.getOriginLevel());
            } else {
                markTaskDTO.setLevel(paper.getLevel());
            }
            markTaskDTO.setSn(markTask.getSecretNumber());
            markTaskDTO.setRedoLevel(paper.getRedoLevel());
            String imgSrc = null;
            String thumbSrc = null;
            if (imageConfig.isImageEnc()) {
                Student student = studentRepo.findByWorkIdAndExamNumberAndTest(paper.getWorkId(), paper.getExamNumber(), String.valueOf(TrialEnum.DEFAULT.getId()));
                imgSrc = systemConfig.getImageUrl(paper.getWorkId(), paper.getSubject().ordinal(), student.getId(), SystemConstant.IMAGE);
                thumbSrc = systemConfig.getThumbUrl(paper.getWorkId(), paper.getSubject().ordinal(), student.getId(), SystemConstant.THUMB);
            } else {
                imgSrc = systemConfig.getImageUrl(markTask.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
                thumbSrc = systemConfig.getThumbUrl(markTask.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
            }
//            String imgSrc = systemConfig.getImageUrl(markTask.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
//            String thumbSrc = systemConfig.getThumbUrl(markTask.getWorkId(), paper.getSubject().toString(), paper.getAreaCode(), paper.getExamNumber());
            markTaskDTO.setThumbSrc(thumbSrc);
            markTaskDTO.setImgSrc(imgSrc);
            markTaskDTO.setSample(paper.isSample());
            markTaskDTO.setMarkByLeader(paper.isMarkByLeader());
            markTaskDTO.setRandomSeqNew(markTask.getRandomSeqNew());
        }
        return markTaskDTO;
    }
}